<?php

/**
 * EtatFichesController : Controller de l'état de validation des fiches
 *
 * web-DPO : Outil de gestion de vos traitements dans le cadre de la
 * réglementation relative à la protection des données personnelles (RGPD)
 *
 * Copyright (c) Libriciel SCOP (https://www.libriciel.fr/)
 *
 * Licensed under the GNU Affero General Public License version 3 License - AGPL v3
 * For full copyright and license information, please see the "LICENSE" file.
 * Redistributions of files must retain the above copyright notice.
 *
 * @copyright   Copyright (c) Libriciel SCOP (https://www.libriciel.fr/)
 * @link        https://www.libriciel.fr/web-dpo/
 * @since       web-DPO v1.0.0
 * @license     [GNU Affero General Public License version 3](http://www.gnu.org/licenses/agpl-3.0.html) - AGPL v3
 * @version     v1.0.0
 * @package     App.Controller
 */

class EtatFichesController extends AppController
{

    public $uses = [
        'Commentaire',
        'EtatFiche',
        'ExtraitRegistre',
        'Fiche',
        'Historique',
        'ModeleExtraitRegistre',
        'Notification',
        'Organisation',
        'Pannel',
        'TraitementRegistre',
        'User',
        'Valeur'
    ];

    /**
     * Envoie ou renvoie d'un traitement en validation et crée les états
     *
     * @access public
     * @created 29/04/2015
     * @version V1.0.0
     */
    public function sendValidation()
    {
        $success = true;
        $this->EtatFiche->begin();

        //On met EtatFiche.actif a false en fonction de l'id
        $success = $success && $this->EtatFiche->updateAllIfFound(
                ['actif' => false],
                [
                    'id' => $this->request->data['EtatFiche']['etatFiche'],
                    'etat_id <>' => EtatFiche::DEMANDE_AVIS
                ]
            ) !== false;

        if ($success == true) {
            $idEncoursValid = $this->EtatFiche->find('first', [
                'conditions' => [
                    'fiche_id' => $this->request->data['EtatFiche']['ficheNum'],
                    'etat_id' => EtatFiche::ENCOURS_VALIDATION,
                    'user_id' => $this->Auth->user('id'),
                    'actif' => false
                ]
            ]);

            if (!empty($idEncoursValid)) {
                $this->EtatFiche->create([
                    'EtatFiche' => [
                        'fiche_id' => $this->request->data['EtatFiche']['ficheNum'],
                        'etat_id' => EtatFiche::VALIDER,
                        'user_id' => $this->Auth->user('id'),
                        'previous_user_id' => $this->Auth->user('id'),
                        'actif' => false
                    ]
                ]);
                $success = $success && false !== $this->EtatFiche->save(null, ['atomic' => false]);

                $messageHistorique = __d('historique', 'historique.valideEnvoieTraitement');

                $success = $success && $this->Commentaire->updateAll([
                        'etat_fiches_id' => $this->EtatFiche->id
                    ], [
                            'etat_fiches_id' => $this->request->data['EtatFiche']['etatFiche']
                        ]
                    ) !== false;

            } else {
                $messageHistorique = __d('historique', 'historique.envoieTraitement');
            }

            if ($success == true) {

                $this->EtatFiche->create([
                    'EtatFiche' => [
                        'fiche_id' => $this->request->data['EtatFiche']['ficheNum'],
                        'etat_id' => EtatFiche::ENCOURS_VALIDATION,
                        'previous_user_id' => $this->Auth->user('id'),
                        'user_id' => $this->request->data['EtatFiche']['destinataire']
                    ]
                ]);
                $success = $success && false !== $this->EtatFiche->save(null, ['atomic' => false]);

                if ($success == true) {
                    $this->Notification->create([
                        'Notification' => [
                            'user_id' => $this->request->data['EtatFiche']['destinataire'],
                            'content' => Notification::VALIDATION_DEMANDEE,
                            'fiche_id' => $this->request->data['EtatFiche']['ficheNum']
                        ]
                    ]);
                    $success = $success && false !== $this->Notification->save(null, ['atomic' => false]);

                    if ($success === true) {
                        $this->Notifications->sendEmail($this->request->data['EtatFiche']['destinataire']);
                    }

                    if ($success == true) {
                        $destinataire = $this->User->find('first', [
                            'conditions' => [
                                'id' => $this->request->data['EtatFiche']['destinataire']
                            ]
                        ]);

                        $this->Historique->create([
                            'Historique' => [
                                'content' => $this->Auth->user('prenom') . ' ' . $this->Auth->user('nom') . ' ' . $messageHistorique . ' ' . $destinataire['User']['prenom'] . ' ' . $destinataire['User']['nom'] . ' ' . __d('historique', 'historique.validation'),
                                'fiche_id' => $this->request->data['EtatFiche']['ficheNum']
                            ]
                        ]);
                        $success = $success && false !== $this->Historique->save(null, ['atomic' => false]);
                    }
                }
            }
        }

        if ($success == true) {
            $this->EtatFiche->commit();
            $this->Session->setFlash(__d('etat_fiche', 'etat_fiche.flashsuccessTraitementEnvoyerValidation'), 'flashsuccess');

            $this->requestAction([
                'controller' => 'pannel',
                'action' => 'supprimerLaNotif',
                $this->request->data['EtatFiche']['ficheNum']
            ]);
        } else {
            $this->EtatFiche->rollback();
            $this->Session->setFlash(__d('default', 'default.flasherrorEnregistrementErreur'), 'flasherror');
        }

        $this->redirect([
            'controller' => 'pannel',
            'action' => 'index'
        ]);
    }

    /**
     * Envoie une fiche en réorientation
     *
     * @access public
     * @created 29/04/2015
     * @version V1.0.0
     */
    public function reorientation()
    {
        $success = true;
        $error = __d('default', 'default.flasherrorEnregistrementErreur');
        $this->EtatFiche->begin();

        $traitement = $this->EtatFiche->find('first', [
            'conditions' => [
                'id' => $this->request->data['EtatFiche']['etatFiche']
            ]
        ]);

        if ($traitement['EtatFiche']['user_id'] == $this->request->data['EtatFiche']['destinataire']) {
            $success = false;
            $error = "Le traitement est déjà chez le valideur concerné";
        }

        if ($success == true) {
            $success = $success && false !== $this->EtatFiche->delete($this->request->data['EtatFiche']['etatFiche']);
            $this->EtatFiche->create([
                'EtatFiche' => [
                    'fiche_id' => $this->request->data['EtatFiche']['ficheNum'],
                    'etat_id' => 2,
                    'previous_user_id' => $this->Auth->user('id'),
                    'user_id' => $this->request->data['EtatFiche']['destinataire']
                ]
            ]);
            $success = $success && false !== $this->EtatFiche->save(null, ['atomic' => false]);
        }

        if ($success == true) {
            $destinataire = $this->User->find('first', [
                'conditions' => [
                    'id' => $this->request->data['EtatFiche']['destinataire']
                ]
            ]);

            $this->Historique->create([
                'Historique' => [
                    'content' => $this->Auth->user('prenom') . ' ' . $this->Auth->user('nom') . ' réoriente la fiche vers ' . $destinataire['User']['prenom'] . ' ' . $destinataire['User']['nom'] . ' pour validation',
                    'fiche_id' => $this->request->data['EtatFiche']['ficheNum']
                ]
            ]);
            $success = $success && false !== $this->Historique->save(null, ['atomic' => false]);

            if ($success == true) {
                $success = $success && false !== $this->Notifications->del(2, $this->request->data['EtatFiche']['ficheNum']);

                if ($success == true) {
                    $this->Notification->create([
                        'Notification' => [
                            'user_id' => $this->request->data['EtatFiche']['destinataire'],
                            'content' => Notification::VALIDATION_DEMANDEE,
                            'fiche_id' => $this->request->data['EtatFiche']['ficheNum']
                        ]
                    ]);
                    $success = $success && false !== $this->Notification->save(null, ['atomic' => false]);
                }
            }
        }

        if ($success == true) {
            $this->EtatFiche->commit();
            $this->Session->setFlash(__d('etat_fiche', 'etat_fiche.flashsuccessTraitementRedirige'), 'flashsuccess');

            $this->requestAction([
                'controller' => 'pannel',
                'action' => 'supprimerLaNotif',
                $this->request->data['EtatFiche']['ficheNum']
            ]);
        } else {
            $this->EtatFiche->rollback();
            $this->Session->setFlash($error, 'flasherror');
        }

        $this->redirect($this->Referers->get());
    }

    /**
     * Gère le refus de validation et le commentaire associé
     *
     * @access public
     * @created 29/04/2015
     * @version V1.0.0
     */
    public function refuse()
    {
        if ($this->request->is('post') || $this->request->is('put')) {
            if ('Cancel' === Hash::get($this->request->data, 'submit')) {
                $this->redirect($this->Referers->get());
            }

            if (empty($this->request->data('EtatFiche.commentaireRepondre'))) {
                $this->Session->setFlash(__d('default', 'default.flasherrorChampVide'), 'flasherror');
                $this->redirect($this->Referers->get());
            }

            $success = true;
            $this->EtatFiche->begin();

            $idEncoursValid = $this->EtatFiche->find('first', [
                'conditions' => [
                    'EtatFiche.fiche_id' => $this->request->data['EtatFiche']['ficheNum'],
                    'EtatFiche.etat_id' => EtatFiche::ENCOURS_VALIDATION,
                    'EtatFiche.actif' => true
                ],
                'fields' => 'id'
            ]);
            $id = $idEncoursValid['EtatFiche']['id'];
            $this->EtatFiche->id = $id;

            $infoTraitement = $this->Fiche->find('first', [
                'conditions' => [
                    'Fiche.id' => $this->request->data['EtatFiche']['ficheNum']
                ],
                'fields' => ['id'],
                'contain' => [
                    'User' => array('id')
                ]
            ]);
            $idFiche = $infoTraitement['Fiche']['id'];
            $idDestinataire = $infoTraitement['User']['id'];

            $success = $success && $this->EtatFiche->updateAll([
                    'actif' => false
                ], [
                        'id' => $id
                    ]
                ) !== false;

            if ($success == true) {
                $this->EtatFiche->create([
                    'EtatFiche' => [
                        'etat_id' => EtatFiche::REFUSER,
                        'previous_user_id' => $this->Auth->user('id'),
                        'user_id' => $idDestinataire,
                        'fiche_id' => $idFiche
                    ]
                ]);
                $success = $success && false !== $this->EtatFiche->save(null, ['atomic' => false]);

                if ($success == true) {
                    $this->Commentaire->create([
                        'Commentaire' => [
                            'etat_fiches_id' => $this->EtatFiche->id,
                            'content' => $this->request->data['EtatFiche']['commentaireRepondre'],
                            'user_id' => $this->Auth->user('id'),
                            'destinataire_id' => $idDestinataire
                        ]
                    ]);
                    $success = $success && false !== $this->Commentaire->save(null, ['atomic' => false]);

                    if ($success == true) {
                        $this->Historique->create([
                            'Historique' => [
                                'content' => $this->Auth->user('prenom') . ' ' . $this->Auth->user('nom') . ' refuse le traitement',
                                'fiche_id' => $this->request->data['EtatFiche']['ficheNum']
                            ]
                        ]);
                        $success = $success && false !== $this->Historique->save(null, ['atomic' => false]);

                        if ($success == true) {
                            $this->Notification->create([
                                'Notification' => [
                                    'user_id' => $idDestinataire,
                                    'content' => EtatFiche::REFUSER,
                                    'fiche_id' => $idFiche
                                ]
                            ]);
                            $success = $success && false !== $this->Notification->save(null, ['atomic' => false]);

                            if ($success === true) {
                                $this->Notifications->sendEmail($idDestinataire);
                            }
                        }
                    }
                }
            }

            if ($success == true) {
                $this->EtatFiche->commit();
                $this->Session->setFlash(__d('etat_fiche', 'etat_fiche.flashsuccessTraitementRefuse'), 'flashsuccess');

                $this->requestAction([
                    'controller' => 'pannel',
                    'action' => 'supprimerLaNotif',
                    $idFiche
                ]);
            } else {
                $this->EtatFiche->rollback();
                $this->Session->setFlash(__d('default', 'default.flasherrorEnregistrementErreur'), 'flasherror');
            }

            $this->redirect($this->Referers->get());
        }
    }

    /**
     * Gère l'envoie en consultation
     *
     * @access public
     * @created 29/04/2015
     * @version V1.0.0
     */
    public function askAvis()
    {
        $success = true;
        $this->EtatFiche->begin();

        $count = $this->EtatFiche->find('count', [
            'conditions' => [
                'fiche_id' => $this->request->data['EtatFiche']['ficheNum'],
                'etat_id' => EtatFiche::DEMANDE_AVIS,
                'user_id' => $this->request->data['EtatFiche']['destinataire'],
                'previous_user_id' => $this->Auth->user('id'),
                'actif' => true
            ]
        ]);

        if ($count > 0) {
            $this->Session->setFlash(__d('etat_fiche', 'etat_fiche.flashwarningTraitementDejaAttenteUser'), 'flashwarning');
            $this->redirect([
                'controller' => 'pannel',
                'action' => 'index'
            ]);
        } else {
            /* Si le traitement est en attente de validation et qu'on l'envoie
             * pour consultation on se change pas l'état "actif" du traitement 
             * pour qu'il reste en attende de validation
             */
            $traitementEnCoursValidation = $this->EtatFiche->find('first', [
                'conditions' => [
                    'fiche_id' => $this->request->data['EtatFiche']['ficheNum'],
                    'etat_id' => EtatFiche::ENCOURS_VALIDATION,
                    'user_id' => $this->Auth->user('id'),
                    'actif' => true
                ]
            ]);

            $reorientationDemandeAvis = $this->EtatFiche->find('first', [
                'conditions' => [
                    'id' => $this->request->data['EtatFiche']['etatFiche'],
                    'fiche_id' => $this->request->data['EtatFiche']['ficheNum'],
                    'etat_id' => EtatFiche::DEMANDE_AVIS,
                    'actif' => true
                ]
            ]);

            if (empty($traitementEnCoursValidation) && empty($reorientationDemandeAvis)) {
                $success = $success && $this->EtatFiche->updateAll([
                        'actif' => false
                    ], [
                            'id' => $this->request->data['EtatFiche']['etatFiche']
                        ]
                    ) !== false;
            }

            if ($success == true) {
                if (empty($reorientationDemandeAvis)) {
                    $this->EtatFiche->create([
                        'EtatFiche' => [
                            'etat_id' => EtatFiche::DEMANDE_AVIS,
                            'previous_user_id' => $this->Auth->user('id'),
                            'user_id' => $this->request->data['EtatFiche']['destinataire'],
                            'previous_etat_id' => $this->request->data['EtatFiche']['etatFiche'],
                            'fiche_id' => $this->request->data['EtatFiche']['ficheNum']
                        ]
                    ]);
                    $success = $success && false !== $this->EtatFiche->save(null, ['atomic' => false]);
                } else {
                    $success = $success && $this->EtatFiche->updateAll([
                            'user_id' => $this->request->data['EtatFiche']['destinataire']
                        ], [
                                'id' => $this->request->data['EtatFiche']['etatFiche']
                            ]
                        ) !== false;
                }

                if ($success == true) {
                    $destinataire = $this->User->find('first', [
                        'conditions' => [
                            'id' => $this->request->data['EtatFiche']['destinataire']
                        ]
                    ]);

                    if (!empty($reorientationDemandeAvis)) {
                        $precedentDestinataire = $this->User->find('first', [
                            'conditions' => [
                                'id' => $reorientationDemandeAvis['EtatFiche']['user_id']
                            ]
                        ]);

                        $this->Historique->create([
                            'Historique' => [
                                'content' => $this->Auth->user('prenom') . ' ' . $this->Auth->user('nom') . ' ' . __d('historique', 'historique.annulerDemandeAvis') . ' ' . $precedentDestinataire['User']['prenom'] . ' ' . $precedentDestinataire['User']['nom'],
                                'fiche_id' => $this->request->data['EtatFiche']['ficheNum']
                            ]
                        ]);
                        $success = $success && false !== $this->Historique->save(null, ['atomic' => false]);
                    }

                    $this->Historique->create([
                        'Historique' => [
                            'content' => $this->Auth->user('prenom') . ' ' . $this->Auth->user('nom') . ' ' . __d('historique', 'historique.demandeAvis') . ' ' . $destinataire['User']['prenom'] . ' ' . $destinataire['User']['nom'],
                            'fiche_id' => $this->request->data['EtatFiche']['ficheNum']
                        ]
                    ]);
                    $success = $success && false !== $this->Historique->save(null, ['atomic' => false]);

                    if ($success == true) {
                        $this->Notification->create([
                            'Notification' => [
                                'user_id' => $this->request->data['EtatFiche']['destinataire'],
                                'content' => Notification::DEMANDE_AVIS,
                                'fiche_id' => $this->request->data['EtatFiche']['ficheNum']
                            ]
                        ]);
                        $success = $success && false !== $this->Notification->save(null, ['atomic' => false]);

                        if ($success === true) {
                            $this->Notifications->sendEmail($this->request->data['EtatFiche']['destinataire']);
                        }
                    }
                }
            }

            if ($success == true) {
                $this->EtatFiche->commit();
                $this->Session->setFlash(__d('etat_fiche', 'etat_fiche.flashsuccessTraitementEnvoyerAvis'), 'flashsuccess');

                $this->requestAction([
                    'controller' => 'pannel',
                    'action' => 'supprimerLaNotif',
                    $this->request->data['EtatFiche']['ficheNum']
                ]);
            } else {
                $this->EtatFiche->rollback();
                $this->Session->setFlash(__d('default', 'default.flasherrorEnregistrementErreur'), 'flasherror');
            }

            $this->redirect([
                'controller' => 'pannel',
                'action' => 'index'
            ]);
        }
    }

    public function answerAvis()
    {
        if ($this->request->is('post') || $this->request->is('put')) {
            if ('Cancel' === Hash::get($this->request->data, 'submit')) {
                $this->redirect($this->Referers->get());
            }

            if (empty($this->request->data('EtatFiche.commentaireRepondre'))) {
                $this->Session->setFlash(__d('default', 'default.flasherrorChampVide'), 'flasherror');
                $this->redirect($this->Referers->get());
            }

            $success = true;
            $this->EtatFiche->begin();

            $traitementEnReponse = $this->EtatFiche->find('first', [
                'conditions' => [
                    'id' => $this->request->data['EtatFiche']['etatFiche']
                ]
            ]);

            $previousTraitement = $this->EtatFiche->find('first', [
                'conditions' => [
                    'id' => $traitementEnReponse['EtatFiche']['previous_etat_id']
                ]
            ]);

            $traitements = $this->EtatFiche->find('all', [
                'conditions' => [
                    'id <>' => [
                        $traitementEnReponse['EtatFiche']['id'],
                        $traitementEnReponse['EtatFiche']['previous_etat_id']
                    ],
                    'fiche_id' => $this->request->data['EtatFiche']['ficheNum']
                ]
            ]);

            switch ($previousTraitement['EtatFiche']['etat_id']) {
                case EtatFiche::ENCOURS_REDACTION:
                    $etat_fiches_id = $traitementEnReponse['EtatFiche']['previous_etat_id'];
                    break;
                case EtatFiche::ENCOURS_VALIDATION:
                    if ($previousTraitement['EtatFiche']['actif'] == true) {
                        $etat_fiches_id = $traitementEnReponse['EtatFiche']['previous_etat_id'];
                        $actif = false;
                    }
                    break;
                case EtatFiche::REPLACER_REDACTION:
                    $etat_fiches_id = $traitementEnReponse['EtatFiche']['previous_etat_id'];
                    $actif = true;
                    break;
                case EtatFiche::TRAITEMENT_INITIALISE_RECU:
                    $etat_fiches_id = $traitementEnReponse['EtatFiche']['previous_etat_id'];
                    $actif = true;
                    break;
                default:
                    break;
            }

            if (empty($traitements)) {
                $actif = true;
            } else {
                foreach ($traitements as $traitement) {
                    if ($traitement['EtatFiche']['etat_id'] == EtatFiche::ENCOURS_VALIDATION && $traitement['EtatFiche']['actif'] == true) {
                        $actif = false;
                    }

                    if ($traitement['EtatFiche']['etat_id'] == EtatFiche::VALIDER && $traitement['EtatFiche']['actif'] == false) {
                        $etat_fiches_id = $traitement['EtatFiche']['id'];
                        $actif = false;
                    }

                    if ($traitement['EtatFiche']['etat_id'] == EtatFiche::ENCOURS_VALIDATION && $traitement['EtatFiche']['actif'] == true) {
                        foreach ($traitements as $fiche) {
                            if ($fiche['EtatFiche']['etat_id'] == EtatFiche::ENCOURS_REDACTION && $fiche['EtatFiche']['actif'] == false) {
                                $etat_fiches_id = $fiche['EtatFiche']['id'];
                                $actif = false;
                            }

                            if ($fiche['EtatFiche']['etat_id'] == EtatFiche::VALIDER && $fiche['EtatFiche']['actif'] == false) {
                                $etat_fiches_id = $fiche['EtatFiche']['id'];
                                $actif = false;
                            }
                        }
                    }

                    //@todo
                    foreach ($traitements as $fiche) {
                        if ($previousTraitement['EtatFiche']['etat_id'] == EtatFiche::REPONSE_AVIS && $previousTraitement['EtatFiche']['actif'] == false && $fiche['EtatFiche']['etat_id'] == EtatFiche::ENCOURS_REDACTION) {
                            $etat_fiches_id = $fiche['EtatFiche']['id'];
                            $actif = true;
                        }

                        if ($previousTraitement['EtatFiche']['etat_id'] == EtatFiche::REPONSE_AVIS && $previousTraitement['EtatFiche']['actif'] == false && $fiche['EtatFiche']['etat_id'] == EtatFiche::REPLACER_REDACTION) {
                            $etat_fiches_id = $fiche['EtatFiche']['id'];
                            $actif = true;
                        }
                    }
                }
            }

            $success = $success && $this->EtatFiche->updateAll([
                    'actif' => false
                ], [
                        'id' => $this->request->data['EtatFiche']['etatFiche']
                    ]
                ) !== false;

            $this->EtatFiche->create([
                'EtatFiche' => [
                    'fiche_id' => $this->request->data['EtatFiche']['ficheNum'],
                    'etat_id' => EtatFiche::REPONSE_AVIS,
                    'user_id' => $this->request->data['EtatFiche']['previousUserId'],
                    'previous_user_id' => $this->Auth->user('id'),
                    'actif' => $actif
                ]
            ]);
            $success = $success && false !== $this->EtatFiche->save(null, ['atomic' => false]);

            if ($success == true) {
                $this->Commentaire->create([
                    'Commentaire' => [
                        'etat_fiches_id' => $etat_fiches_id,
                        'content' => $this->request->data['EtatFiche']['commentaireRepondre'],
                        'user_id' => $this->Auth->user('id'),
                        'destinataire_id' => $this->request->data['EtatFiche']['previousUserId']
                    ]
                ]);
                $success = $success && false !== $this->Commentaire->save(null, ['atomic' => false]);

                if ($success == true) {
                    $destinataire = $this->User->find('first', [
                        'conditions' => [
                            'id' => $this->request->data['EtatFiche']['previousUserId']
                        ]
                    ]);

                    $this->Historique->create([
                        'Historique' => [
                            'content' => $this->Auth->user('prenom') . ' ' . $this->Auth->user('nom') . ' ' . __d('historique', 'historique.repondDemandeAvis') . ' ' . $destinataire['User']['prenom'] . ' ' . $destinataire['User']['nom'],
                            'fiche_id' => $this->request->data['EtatFiche']['ficheNum']
                        ]
                    ]);
                    $success = $success && false !== $this->Historique->save(null, ['atomic' => false]);

                    if ($success == true) {
                        $this->Notification->create([
                            'Notification' => [
                                'user_id' => $this->request->data['EtatFiche']['previousUserId'],
                                'content' => Notification::COMMENTAIRE_AJOUTE,
                                'fiche_id' => $this->request->data['EtatFiche']['ficheNum']
                            ]
                        ]);
                        $success = $success && false !== $this->Notification->save(null, ['atomic' => false]);

                        if ($success === true) {
                            $this->Notifications->sendEmail($this->request->data['EtatFiche']['previousUserId']);
                        }
                    }
                }
            }

            if ($success == true) {
                $this->EtatFiche->commit();
                $this->Session->setFlash(__d('etat_fiche', 'etat_fiche.flashsuccessCommentaireAjouter'), 'flashsuccess');

                $this->requestAction([
                    'controller' => 'pannel',
                    'action' => 'supprimerLaNotif',
                    $this->request->data['EtatFiche']['ficheNum']
                ]);
            } else {
                $this->EtatFiche->rollback();
                $this->Session->setFlash(__d('default', 'default.flasherrorEnregistrementErreur'), 'flasherror');
            }

            $this->redirect($this->Referers->get());
        }
    }

    /**
     * Fonction permettant de répondre à la demande de consultation
     */
    public function repondreCommentaire()
    {
        if ($this->request->is('POST')) {
            $success = true;
            $this->Commentaire->begin();

            $this->Commentaire->create([
                'Commentaire' => [
                    'etat_fiches_id' => $this->request->data['EtatFiche']['etat_fiche_id'],
                    'content' => $this->request->data['EtatFiche']['commentaire'],
                    'user_id' => $this->Auth->user('id'),
                    'destinataire_id' => $this->Auth->user('id')
                ]
            ]);
            $success = $success && false !== $this->Commentaire->save(null, ['atomic' => false]);

            if ($success == true) {

                $this->Historique->create([
                    'Historique' => [
                        'content' => $this->Auth->user('prenom') . ' ' . $this->Auth->user('nom') . ' répond à l\'avis',
                        'fiche_id' => $this->request->data['EtatFiche']['fiche_id']
                    ]
                ]);
                $success = $success && false !== $this->Historique->save(null, ['atomic' => false]);

                if ($success == true) {
                    foreach (json_decode($this->request->data['EtatFiche']['idUserCommentaire']) as $idUserCommentaire) {
                        $this->Notification->create([
                            'Notification' => [
                                'user_id' => $idUserCommentaire,
                                'content' => Notification::COMMENTAIRE_AJOUTE,
                                'fiche_id' => $this->request->data['EtatFiche']['fiche_id']
                            ]
                        ]);
                        $success = $success && false !== $this->Notification->save(null, ['atomic' => false]);
                    }
                }
            }

            if ($success == true) {
                $this->EtatFiche->commit();
                $this->Session->setFlash(__d('etat_fiche', 'etat_fiche.flashsuccessCommentaireEnregistre'), 'flashsuccess');
            } else {
                $this->EtatFiche->rollback();
                $this->Session->setFlash(__d('default', 'default.flasherrorEnregistrementErreur'), 'flasherror');
            }

            $this->redirect($this->referer());
        }
    }

    /**
     * Gère la remise en rédaction d'une fiche refusée
     *
     * @param int $id
     *
     * @access public
     * @created 29/04/2015
     * @version V1.0.0
     */
    public function relaunch($id)
    {
        if (!$id) {
            $this->Session->setFlash(__d('default', 'default.flasherrorTraitementInexistant'), 'flasherror');
            $this->redirect([
                'controller' => 'pannel',
                'action' => 'index'
            ]);
        }

        $success = true;
        $this->EtatFiche->begin();

        $this->requestAction([
            'controller' => 'pannel',
            'action' => 'supprimerLaNotif',
            $id
        ]);

        $success = $success && $this->EtatFiche->updateAll([
                'actif' => false
            ], [
                    'fiche_id' => $id
                ]
            ) !== false;

        if ($success == true) {
            $this->EtatFiche->create([
                'EtatFiche' => [
                    'fiche_id' => $id,
                    'etat_id' => 8,
                    'previous_user_id' => $this->Auth->user('id'),
                    'user_id' => $this->Auth->user('id')
                ]
            ]);
            $success = $success && false !== $this->EtatFiche->save(null, ['atomic' => false]);

            if ($success == true) {
                $this->Historique->create([
                    'Historique' => [
                        'content' => $this->Auth->user('prenom') . ' ' . $this->Auth->user('nom') . ' replace le traitement en rédaction',
                        'fiche_id' => $id
                    ]
                ]);
                $success = $success && false !== $this->Historique->save(null, ['atomic' => false]);
            }
        }

        if ($success == true) {
            $this->EtatFiche->commit();
            $this->Session->setFlash(__d('etat_fiche', 'etat_fiche.flashsuccessTraitementReplacerRedaction'), 'flashsuccess');
        } else {
            $this->EtatFiche->rollback();
            $this->Session->setFlash(__d('default', 'default.flasherrorEnregistrementErreur'), 'flasherror');
        }

        $this->redirect([
            'controller' => 'pannel',
            'action' => 'index'
        ]);
    }

    /**
     * Gère l'envoie en validation au DPO
     *
     * @param int $id
     *
     * @access public
     * @created 29/04/2015
     * @version V1.0.0
     */
    public function dpoValid($id)
    {
        $success = true;
        $this->EtatFiche->begin();

        $traitement = $this->EtatFiche->find('first', [
            'conditions' => [
                'EtatFiche.fiche_id' => $id,
                'EtatFiche.user_id' => $this->Auth->user('id'),
                'EtatFiche.actif' => true
            ]
        ]);

        $success = $success && $this->EtatFiche->updateAll([
                'actif' => false
            ], [
                    'id' => $traitement['EtatFiche']['id']
                ]
            ) !== false;

        if ($success == true) {
            $dpo = $this->Organisation->find('first', [
                'conditions' => [
                    'Organisation.id' => $this->Session->read('Organisation.id')
                ],
                'fields' => 'dpo'
            ]);
            $idDpo = $dpo['Organisation']['dpo'];

            if ($idDpo != null) {
                if (!empty($traitement)) {
                    if ($traitement['EtatFiche']['etat_id'] !== EtatFiche::REPLACER_REDACTION && $traitement['EtatFiche']['etat_id'] !== EtatFiche::ENCOURS_REDACTION) {
                        $this->EtatFiche->create([
                            'EtatFiche' => [
                                'fiche_id' => $id,
                                'etat_id' => EtatFiche::VALIDER,
                                'user_id' => $this->Auth->user('id'),
                                'previous_user_id' => $this->Auth->user('id'),
                                'actif' => false
                            ]
                        ]);
                        $success = $success && false !== $this->EtatFiche->save(null, ['atomic' => false]);

                        if ($success == true) {
                            $success = $success && $this->Commentaire->updateAll([
                                    'etat_fiches_id' => $this->EtatFiche->id
                                ], [
                                        'etat_fiches_id' => $traitement['EtatFiche']['id']
                                    ]
                                ) !== false;
                        }
                    }
                }

                $this->EtatFiche->create([
                    'EtatFiche' => [
                        'fiche_id' => $id,
                        'etat_id' => EtatFiche::ENCOURS_VALIDATION,
                        'previous_user_id' => $this->Auth->user('id'),
                        'user_id' => $idDpo
                    ]
                ]);
                $success = $success && false !== $this->EtatFiche->save(null, ['atomic' => false]);

                if ($success == true) {
                    $this->Historique->create([
                        'Historique' => [
                            'content' => $this->Auth->user('prenom') . ' ' . $this->Auth->user('nom') . ' ' . __d('historique', 'historique.valideEnvoieTraitementDPO'),
                            'fiche_id' => $id
                        ]
                    ]);
                    $success = $success && false !== $this->Historique->save(null, ['atomic' => false]);

                    if ($success == true) {
                        $this->Notification->create([
                            'Notification' => [
                                'user_id' => $dpo['Organisation']['dpo'],
                                'content' => Notification::VALIDATION_DEMANDEE,
                                'fiche_id' => $id
                            ]
                        ]);
                        $success = $success && false !== $this->Notification->save(null, ['atomic' => false]);

                        if ($success === true) {
                            $this->Notifications->sendEmail($dpo['Organisation']['dpo']);
                        }
                    }
                }
            }
        }

        if ($success == true) {
            $this->EtatFiche->commit();
            $this->Session->setFlash(__d('etat_fiche', 'etat_fiche.flashsuccessTraitementEnvoyerDPO'), 'flashsuccess');

            $this->requestAction([
                'controller' => 'pannel',
                'action' => 'supprimerLaNotif',
                $id
            ]);
        } else {
            $this->EtatFiche->rollback();
            $this->Session->setFlash(__d('default', 'default.flasherrorEnregistrementErreur'), 'flasherror');
        }

        $this->redirect([
            'controller' => 'pannel',
            'action' => 'index'
        ]);
    }

    /**
     * Insère dans le registre le traiment
     * Gère la validation du DPO
     *
     * @param int $id
     *
     * @access public
     * @created 29/04/2015
     * @version V1.0.0
     */
    public function insertRegistre($id, $numero = null, $norme = null)
    {
        if (empty($id)) {
            throw new NotFoundException();
        }

        if (true !== $this->Droits->authorized($this->Droits->isDpo())) {
            throw new ForbiddenException(__d('default', 'default.flasherrorPasDroitPage'));
        }

        $organisation_id = $this->Session->read('Organisation.id');

        if ($numero == null || $numero === 'null') {
            $saveNumeroRegistre = true;

            $organisation = $this->Organisation->find('first', [
                'conditions' => [
                    'Organisation.id' => $organisation_id
                ],
                'fields' => [
                    'Organisation.id',
                    'Organisation.prefixenumero',
                    'Organisation.numeroregistre'
                ]
            ]);

            $numeroregistre = $organisation['Organisation']['numeroregistre'] + 1;
            $numero = $organisation['Organisation']['prefixenumero'] . $numeroregistre;
        } else {
            $saveNumeroRegistre = false;
        }

        $messageError = __d('default', 'default.flasherrorEnregistrementErreur');
        $success = true;
        $this->EtatFiche->begin();

        $success = $success && $this->insertValueEntiteInFiche($organisation_id, $id);

        if ($success === true) {
            $this->Fiche->id = $id;
            $record = [
                'id' => $id,
                'numero' => $numero,
                'norme_id' => $norme
            ];
            $success = $success && $this->Fiche->save($record, ['atomic' => false]) !== false;
        }

        if ($success === true) {
            $idEncoursValid = $this->EtatFiche->find('first', [
                'conditions' => [
                    'EtatFiche.fiche_id' => $id,
                    'EtatFiche.etat_id' => EtatFiche::ENCOURS_VALIDATION,
                    'EtatFiche.actif' => true
                ],
                'fields' => ['id'],
                'contain' => [
                    'Fiche' => [
                        'user_id'
                    ]
                ]
            ]);

            $success = $success && $this->EtatFiche->updateAll([
                    'actif' => false
                ], [
                        'id' => $idEncoursValid['EtatFiche']['id']
                    ]
                ) !== false;

            if ($success === true) {
                if (!empty($idEncoursValid)) {
                    $this->EtatFiche->create([
                        'EtatFiche' => [
                            'fiche_id' => $id,
                            'etat_id' => EtatFiche::VALIDER_DPO,
                            'user_id' => $this->Auth->user('id'),
                            'previous_user_id' => $this->Auth->user('id'),
                            'actif' => true
                        ]
                    ]);
                    $success = $success && false !== $this->EtatFiche->save(null, ['atomic' => false]);

                    if ($success === true) {
                        $this->Notification->create([
                            'Notification' => [
                                'user_id' => $idEncoursValid['Fiche']['user_id'],
                                'content' => Notification::TRAITEMENT_VALIDE,
                                'fiche_id' => $id
                            ]
                        ]);
                        $success = $success && false !== $this->Notification->save(null, ['atomic' => false]);

                        if ($success === true) {
                            $this->Notifications->sendEmail($idEncoursValid['Fiche']['user_id']);
                        }

                        if ($success === true) {
                            $this->Historique->create([
                                'Historique' => [
                                    'content' => $this->Auth->user('prenom') . ' ' . $this->Auth->user('nom') . ' valide le traitement et l\'insère au registre',
                                    'fiche_id' => $id
                                ]
                            ]);
                            $success = $success && false !== $this->Historique->save(null, ['atomic' => false]);
                        }

                        if ($success === true && $saveNumeroRegistre === true) {
                            $this->Organisation->id = $organisation_id;
                            $record = [
                                'id' => $organisation_id,
                                'numeroregistre' => $numeroregistre
                            ];
                            $success = $success && $this->Organisation->save($record, ['atomic' => false]) !== false;
                        }
                    }
                }
            }
        } else {
            $messageError = sprintf("Le numéro d'enregistrement %s est déja présent au registre. Merci de modifier le numéro d'enregistrement.", $numero);
        }

        if ($success === true) {
            $this->EtatFiche->commit();
            $this->Session->setFlash(__d('etat_fiche', 'etat_fiche.flashsuccessTraitementEngregistreRegistre'), 'flashsuccess');

            $this->requestAction([
                'controller' => 'pannel',
                'action' => 'supprimerLaNotif',
                $idEncoursValid['Fiche']['id']
            ]);

            $ficheRT = $this->Fiche->find('first', [
                'conditions' => [
                    'id' => $id
                ],
                'fields' => [
                    'rt_externe'
                ]
            ]);

            $action = 'registreActivite';
            if ($ficheRT['Fiche']['rt_externe'] === true) {
                $action = 'registreSoustraitance';
            }

            $this->redirect([
                'controller' => 'registres',
                'action' => $action
            ]);
        } else {
            $this->EtatFiche->rollback();
            $this->Session->setFlash($messageError, 'flasherror');

            $this->redirect($this->Referers->get());
        }
    }

    protected function insertValueEntiteInFiche($organisation_id, $fiche_id)
    {
        $success = true;

        $entite = $this->Organisation->find('first', [
            'conditions' => [
                'id' => $organisation_id
            ],
            'fields' => [
                'raisonsociale',
                'telephone',
                'fax',
                'adresse',
                'email',
                'sigle',
                'siret',
                'ape',
                'civiliteresponsable',
                'nomresponsable',
                'prenomresponsable',
                'emailresponsable',
                'telephoneresponsable',
                'fonctionresponsable',
                'dpo',
                'numerodpo'
            ]
        ]);

        $userDPO = $this->User->find('first', [
            'conditions' => [
                'id' => $entite['Organisation']['dpo']
            ],
            'fields' => [
                'nom_complet',
                'email',
                'telephonefixe',
                'telephoneportable'
            ]
        ]);

        unset($entite['Organisation']['dpo']);

        $ficheIsRT = $this->Fiche->find('first', [
            'conditions' => [
                'Fiche.id' => $fiche_id
            ],
            'fields' => [
                'rt_externe'
            ]
        ]);

        $prefixChampName = 'rt_organisation_';
        if ($ficheIsRT['Fiche']['rt_externe'] === true) {
            $prefixChampName = 'st_organisation_';
        }

        $entite = Hash::extract($entite, 'Organisation');
        $keyEntite = array_keys($entite);

        foreach ($keyEntite as $val) {
            if ($success === true) {
                if (!empty($entite[$val])) {
                    $this->Valeur->create([
                        'fiche_id' => $fiche_id,
                        'valeur' => $entite[$val],
                        'champ_name'=> $prefixChampName.$val
                    ]);
                    $success = $success && false !== $this->Valeur->save(null, ['atomic' => false]);
                }
            }
        }

        if ($success === true) {
            $userDPO = Hash::extract($userDPO, 'User');
            $keyUserDPO = array_keys($userDPO);

            foreach ($keyUserDPO as $val) {
                if ($success === true) {
                    if (!empty($userDPO[$val])) {
                        if ($val === 'nom_complet') {
                            $userDPO['dpo'] = $userDPO[$val];
                            unset($userDPO[$val]);
                            $val = 'dpo';
                            $champ_name = 'nom_complet_dpo';
                        }

                        if ($val === 'email') {
                            $userDPO['emailDpo'] = $userDPO[$val];
                            unset($userDPO[$val]);
                            $val = 'emailDpo';
                            $champ_name = 'email_dpo';
                        }

                        if ($val === 'telephoneportable') {
                            $userDPO['portableDpo'] = $userDPO[$val];
                            unset($userDPO[$val]);
                            $val = 'portableDpo';
                            $champ_name = 'telephoneportable_dpo';
                        }

                        if ($val === 'telephonefixe') {
                            $userDPO['fixDpo'] = $userDPO[$val];
                            unset($userDPO[$val]);
                            $val = 'fixDpo';
                            $champ_name = 'telephonefixe_dpo';
                        }

                        $this->Valeur->create([
                            'fiche_id' => $fiche_id,
                            'valeur' => $userDPO[$val],
                            'champ_name'=> $prefixChampName.$champ_name
                        ]);
                        $success = $success && false !== $this->Valeur->save(null, ['atomic' => false]);
                    }
                }
            }
        }

        return $success;
    }

    /**
     * Envoie d'un traitement en cour d'initialisation en rédaction
     *
     * @access public
     * @created 10/12/2019
     * @version V2.0.0
     * @author Théo GUILLON <theo.guillon@libriciel.coop>
     */
    public function sendRedaction()
    {
        $success = true;
        $this->EtatFiche->begin();

        // On change le rédacteur du traitement
        $success = $this->_updateRedacteurTraitementInitialise($this->request->data['EtatFiche']['ficheNum'], $this->request->data['EtatFiche']['destinataire']);

        //On met EtatFiche.actif a false en fonction de l'id
        $success = $success && $this->EtatFiche->updateAllIfFound(
            [
                'actif' => false
            ],
            [
                'id' => $this->request->data['EtatFiche']['etatFiche'],
            ]
        ) !== false;

        if ($success === true) {
            $messageHistorique = __d('historique', 'historique.envoieTraitementInitialiserAuRedacteur');

            $this->EtatFiche->create([
                'EtatFiche' => [
                    'fiche_id' => $this->request->data['EtatFiche']['ficheNum'],
                    'etat_id' => EtatFiche::TRAITEMENT_INITIALISE_RECU,
                    'previous_user_id' => $this->Auth->user('id'),
                    'user_id' => $this->request->data['EtatFiche']['destinataire']
                ]
            ]);
            $success = $success && false !== $this->EtatFiche->save(null, ['atomic' => false]);

            if ($success == true) {
                $this->Notification->create([
                    'Notification' => [
                        'user_id' => $this->request->data['EtatFiche']['destinataire'],
                        'content' => Notification::TRAITEMENT_INITIALISE,
                        'fiche_id' => $this->request->data['EtatFiche']['ficheNum']
                    ]
                ]);
                $success = $success && false !== $this->Notification->save(null, ['atomic' => false]);

                if ($success === true) {
                    $this->Notifications->sendEmail($this->request->data['EtatFiche']['destinataire']);
                }

                if ($success == true) {
                    $destinataire = $this->User->find('first', [
                        'conditions' => [
                            'id' => $this->request->data['EtatFiche']['destinataire']
                        ]
                    ]);

                    $this->Historique->create([
                        'Historique' => [
                            'content' => $this->Auth->user('nom_complet') . ' ' . $messageHistorique . ' ' . $destinataire['User']['prenom'] . ' ' . $destinataire['User']['nom'] . ' ' . __d('historique', 'historique.redaction'),
                            'fiche_id' => $this->request->data['EtatFiche']['ficheNum']
                        ]
                    ]);
                    $success = $success && false !== $this->Historique->save(null, ['atomic' => false]);

                    // @todo
//                    if ($success == true) {
//                        $this->Historique->create([
//                            'Historique' => [
//                                'content' => __d('historique', 'historique.redactionInitialisationTraitement') . ' ' . $destinataire['User']['prenom'] . ' ' . $destinataire['User']['nom'] . ' ' . __d('historique', 'historique.redactiontraitementInitialisationTraitement'),
//                                'fiche_id' => $this->request->data['EtatFiche']['ficheNum']
//                            ]
//                        ]);
//                        $success = $success && false !== $this->Historique->save(null, ['atomic' => false]);
//                    }
                }
            }
        }

        if ($success == true) {
            $this->EtatFiche->commit();
            $this->Session->setFlash(__d('etat_fiche', 'etat_fiche.flashsuccessTraitementEnvoyerRedaction'), 'flashsuccess');

            $this->requestAction([
                'controller' => 'pannel',
                'action' => 'supprimerLaNotif',
                $this->request->data['EtatFiche']['ficheNum']
            ]);
        } else {
            $this->EtatFiche->rollback();
            $this->Session->setFlash(__d('default', 'default.flasherrorEnregistrementErreur'), 'flasherror');
        }

        $this->redirect($this->Referers->get());
    }

    /**
     * Mise à jour du rédacteur du traitement initialisé
     *
     * @param $organisation_id int
     * @return bool
     *
     * @access private
     * @created 03/12/2019
     * @version V1.1.0
     */
    private function _updateRedacteurTraitementInitialise($fiche_id, $destinataire)
    {
        $success = true;

        $values = $this->Valeur->find('all', [
            'conditions' => [
                'fiche_id' => $fiche_id,
                'champ_name' => [
                    'declarantpersonnenom',
                    'declarantpersonneemail',
                    'declarantpersonnefix',
                    'declarantpersonneportable',
                ]
            ]
        ]);

        $user = $this->User->find('first', [
            'conditions' => [
                'id' => $destinataire
            ],
            'fields' => [
                'id',
                'nom_complet_court',
                'email',
                'telephonefixe',
                'telephoneportable',
            ]
        ]);

        foreach ($values as $valeurFiche) {
            $success = $success && false !== $this->Valeur->delete($valeurFiche['Valeur']['id'], ['atomic' => false]);

            if ($success === true) {
                $valeurChamp = null;

                switch ($valeurFiche['Valeur']['champ_name']) {
                    case 'declarantpersonnenom':
                        $valeurChamp = $user['User']['nom_complet_court'];
                        break;

                    case 'declarantpersonneportable':
                        if (!empty($user['User']['telephoneportable'])) {
                            $valeurChamp = $user['User']['telephoneportable'];
                        }
                        break;

                    case 'declarantpersonneemail':
                        $valeurChamp = $user['User']['email'];
                        break;

                    case 'declarantpersonnefix':
                        if (!empty($user['User']['telephonefixe'])) {
                            $valeurChamp = $user['User']['telephonefixe'];
                        }
                        break;

                    default:
                        break;
                }

                if (!empty($valeurChamp)) {
                    $record = [
                        'fiche_id' => $fiche_id,
                        'valeur' => $valeurChamp,
                        'champ_name' => $valeurFiche['Valeur']['champ_name']
                    ];
                    $success = $success && $this->Valeur->save($record, ['atomic' => false]) !== false;
                }
            }
        }

        $this->Fiche->id = $fiche_id;
        $record = [
            'id' => $fiche_id,
            'user_id' => $user['User']['id']
        ];
        $success = $success && $this->Fiche->save($record, ['atomic' => false]) !== false;

        return ($success);
    }

}
