<?php

/**
 * ImportsController
 *
 * web-DPO : Outil de gestion de vos traitements dans le cadre de la
 * réglementation relative à la protection des données personnelles (RGPD)
 *
 * Copyright (c) Libriciel SCOP (https://www.libriciel.fr/)
 *
 * Licensed under the GNU Affero General Public License version 3 License - AGPL v3
 * For full copyright and license information, please see the "LICENSE" file.
 * Redistributions of files must retain the above copyright notice.
 *
 * @copyright   Copyright (c) Libriciel SCOP (https://www.libriciel.fr/)
 * @link        https://www.libriciel.fr/web-dpo/
 * @since       web-DPO v2.0.0
 * @license     [GNU Affero General Public License version 3](http://www.gnu.org/licenses/agpl-3.0.html) - AGPL v3
 * @version     v2.0.0
 * @package     App.Controller
 */

App::uses('CustomXlsxReaderComponent', 'Controller/Component');

use PhpOffice\PhpSpreadsheet;
use PhpOffice\PhpSpreadsheet\Cell\Coordinate;
use PhpOffice\PhpSpreadsheet\Spreadsheet;
use PhpOffice\PhpSpreadsheet\Writer\Xlsx;

class ImportsController extends AppController {

    private function getDirContents($dir, &$results = []) {
        $accept = 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet';

        $files = scandir($dir);

        foreach ($files as $key => $value) {
            $path = realpath($dir . DIRECTORY_SEPARATOR . $value);
            if (!is_dir($path)) {
                $mime = mime_content_type($path);

                if ($mime == $accept) {
                    $results[] = $path;
                }
            } else if ($value != "." && $value != "..") {
                $this->getDirContents($path, $results);
//                $results[] = $path;
            }
        }

        return $results;
    }

    public function index()
    {
        $this->autoRender = false;

        $infoFiles = [];
        $filesErrorNoSheetExists = [];
        $filesErrorTitle = [];
        $filesErrorNoFinalite = [];
        $filesErrorNoLeicite = [];
        $filesErrorNoCategorie = [];


        $path = APP . "Config" . DS . "Schema" . DS . "CreationBase" . DS . "CD13" . DS;

        $files = $this->getDirContents($path);

        foreach ($files as $key => $file) {
            $reader = new \PhpOffice\PhpSpreadsheet\Reader\Xlsx();

            $success = true;

            $reader
                //            ->setLoadSheetsOnly(["TRAIT", "DATA", "SECURITE", "LISTE"])
//                ->setLoadSheetsOnly(["TRAIT"])
                ->setReadDataOnly(true)
//                ->setReadEmptyCells(false)
                ->setReadFilter(new CustomXlsxReaderComponent());
            $spreadsheet = $reader->load($file);

            //        $worksheet = $spreadsheet->getActiveSheet();
//            $worksheet = $spreadsheet->setActiveSheetIndexByName('TRAIT');

            if ($spreadsheet->sheetNameExists('TRAIT')) {
                $worksheet = $spreadsheet->setActiveSheetIndexByName('TRAIT');
            } else {
                if ($spreadsheet->sheetNameExists('TRAITEMENT')){
                    $worksheet = $spreadsheet->setActiveSheetIndexByName('TRAITEMENT');
                } else {
                    $filesErrorNoSheetExists[] = $file;
                }
            }

            $titleFiche = trim($worksheet->getCell('A1')->getValue());

            $valueCreatedFiche = $worksheet->getCell('B2')->getValue();
            if (!empty($valueCreatedFiche)) {
//                $createdFiche = \PhpOffice\PhpSpreadsheet\Shared\Date::excelToDateTimeObject($value)->format('d-m-Y');
                $createdFiche = $valueCreatedFiche;
            } else {
                $date = new DateTime();
                $createdFiche = $date->format('d-m-Y');
            }

            $valueModifiedFiche = $worksheet->getCell('B3')->getValue();
            if (!empty($valueModifiedFiche)) {
//                $modifiedFiche = \PhpOffice\PhpSpreadsheet\Shared\Date::excelToDateTimeObject($value)->format('d-m-Y');
                $modifiedFiche = $valueModifiedFiche;
            } else {
                $date = new DateTime();
                $modifiedFiche = $date->format('d-m-Y');
            }

            $columnFINALITE = null;
            $rowFINALITE = null;
            $columnLICEITE = null;
            $rowLICEITE = null;
            $columnCATEGORIES = null;
            $rowCATEGORIES = null;
            $columnDESIGNATION = null;
            $rowDESIGNATION = null;
            foreach ($worksheet->getRowIterator() as $row) {
                $cellIterator = $row->getCellIterator();

                foreach ($cellIterator as $cell) {
                    if ($cell->getValue() == "FINALITE DU TRAITEMENT EFFECTUE") {
                        $columnFINALITE = $cell->getColumn();
                        $rowFINALITE = $cell->getRow();
                    }

                    if ($cell->getValue() == "LICEITE DU TRAITEMENT : " ||
                        $cell->getValue() == "LICEITE DU TRAITEMENT : Fondement juridique" ||
                        $cell->getValue() == "LICEITE DU TRAITEMENT : article 6-1.c du  RGPD" ||
                        $cell->getValue() == "LICEITE DU TRAITEMENT : article  6-1.e du RGPD" ||
                        $cell->getValue() == "LICEITE DU TRAITEMENT : article 6C" ||
                        $cell->getValue() == "LICEITE DU TRAITEMENT :  RGPD  ART6-e"
                    ) {
                        $columnLICEITE = $cell->getColumn();
                        $rowLICEITE = $cell->getRow();
                    }

                    if ($cell->getValue() == "CATEGORIES DE PERSONNES CONCERNEES" ||
                        $cell->getValue() == "CATEGORIES DE PERSONNES CONCERNEES : "
                    ) {
                        $columnCATEGORIES = $cell->getColumn();
                        $rowCATEGORIES = $cell->getRow();
                    }

                    if ($cell->getValue() == "DESIGNATION DE L'ACTIVITE") {
                        $columnDESIGNATION = $cell->getColumn();
                        $rowDESIGNATION = $cell->getRow();
                    }

//                    $sheet->setCellValueByColumnAndRow

//                    $valueCell = $cell->getValue();
//                    debug($valueCell);
//
//                    $column = $cell->getColumn();
//                    debug($column);
//
//                    $row = $cell->getRow();
//                    debug($row);
//
//                    $test = $cell->getCoordinate();
//                    debug($test);
                }
            }

            if (empty($titleFiche)) {
                $titleFiche = 'A COMPLETER - NOM DU TRAITEMENT VIDE';

                $filesErrorTitle[] = [
                    'file' => $file,
                    'outilnom' => $titleFiche,
                    'warning' => 'empty',
                    'insertValue' => 'A COMPLETER - NOM DU TRAITEMENT VIDE'
                ];
            } else {
                if ($titleFiche == "TITRE DE LA FICHE") {
                    $titleFiche = 'A COMPLETER - NOM DU TRAITEMENT VIDE';

                    $filesErrorTitle[] = [
                        'file' => $file,
                        'outilnom' => $titleFiche,
                        'warning' => 'title == TITRE DE LA FICHE',
                        'insertValue' => 'A COMPLETER - NOM DU TRAITEMENT VIDE'
                    ];
                } else {
                    if ($this->startsWith($titleFiche, "TITRE DE LA FICHE") === true) {
                        $filesErrorTitle[] = [
                            'file' => $file,
                            'outilnom' => $titleFiche,
                            'warning' => 'title start with TITRE DE LA FICHE',
                        ];
                    }
                }
            }

            $lastRow = $worksheet->getHighestRow();

            // FINALITE
            $val_finalite = null;
            $cellFinaliteStart = null;
            $cellFinaliteEnd = null;
            if (!empty($columnFINALITE) &&
                !empty($rowFINALITE) &&
                !empty($columnLICEITE) &&
                !empty($rowLICEITE) &&
                $columnFINALITE === $columnLICEITE
            ) {
                $rowFinaliteStart = $rowFINALITE + 1;
                $rowFinaliteEnd = $rowLICEITE - 1;

                $cellFinaliteStart = $columnFINALITE . $rowFinaliteStart;
                $cellFinaliteEnd = $columnFINALITE . $rowFinaliteEnd;

                if ($cellFinaliteStart == $cellFinaliteEnd) {
                    // Une seule ligne de Finalité
                    $val_finalite = $worksheet->getCell($cellFinaliteStart)->getValue();
                } else {
                    $tmp = null;
                    for ($i = $rowFinaliteStart; $i <= $rowFinaliteEnd; $i++) {
                        $tmp = $worksheet->getCell($columnFINALITE . $i)->getValue();

                        if (!empty($tmp)) {
                            $val_finalite .= $tmp;
                        }
                    }
                }
            } elseif (!empty($columnFINALITE) &&
                !empty($rowFINALITE) &&
                !empty($lastRow)
            ) {
                $rowFinaliteStart = $rowFINALITE + 1;
                $rowFinaliteEnd = $lastRow;

                $tmpFinalite = null;
                for ($i = $rowFinaliteStart; $i <= $rowFinaliteEnd; $i++) {
                    $tmpFinalite = $worksheet->getCell($columnFINALITE . $i)->getValue();

                    if (!empty($tmpFinalite)) {
                        $val_finalite .= $tmpFinalite;
                    }
                }
            }  else {
                $filesErrorNoFinalite[] = [
                    'file' => $file,
                    'error' => 'ERROR FINALITE - NO FILE IN INSERT',
                ];
                $success = false;
            }

            if (empty($val_finalite) && $success === true) {
                $val_finalite = 'A COMPLETER - FINALITE VIDE';

                $filesErrorNoFinalite[] = [
                    'file' => $file,
                    'warning' => 'NO FINALITE',
                    'insertValue' => 'A COMPLETER - FINALITE VIDE'
                ];
            }

            // LEICITE
            $val_leicite = null;
            $cellLeiciteStart = null;
            $cellLeiciteEnd = null;
            if (!empty($columnLICEITE) &&
                !empty($rowLICEITE) &&
                !empty($columnCATEGORIES) &&
                !empty($rowCATEGORIES) &&
                $columnLICEITE === $columnCATEGORIES
            ) {
                $rowLeiciteStart = $rowLICEITE + 1;
                $rowLeiciteEnd = $rowCATEGORIES - 1;

                $cellLeiciteStart = $columnLICEITE . $rowLeiciteStart;
                $cellLeiciteEnd = $columnLICEITE . $rowLeiciteEnd;

                if ($cellLeiciteStart == $cellLeiciteEnd) {
                    // Une seule ligne de Leicite
                    $val_leicite = $worksheet->getCell($cellLeiciteStart)->getValue();
                } else {
                    $tmpLeicite = null;
                    for ($i = $rowLeiciteStart; $i <= $rowLeiciteEnd; $i++) {
                        $tmpLeicite = $worksheet->getCell($columnLICEITE . $i)->getValue();

                        if (!empty($tmpLeicite)) {
                            $val_leicite .= $tmpLeicite;
                        }
                    }
                }
            } else {
                $filesErrorNoLeicite[] = [
                    'file' => $file,
                    'error' => 'ERROR LEICITE - NO FILE IN INSERT',
                    '$columnLICEITE' => $columnLICEITE,
                    '$rowLICEITE' => $rowLICEITE,
                    '$columnCATEGORIES' => $columnCATEGORIES,
                    '$rowCATEGORIES' => $rowCATEGORIES,
                ];
                $success = false;
            }

            if (empty($val_leicite) && $success === true) {
                $val_leicite = 'A COMPLETER - LEICITE VIDE';

                $filesErrorNoLeicite[] = [
                    'file' => $file,
                    'warning' => 'NO LEICITE',
                    'insertValue' => 'A COMPLETER - LEICITE VIDE'
                ];
            }

            // CATEGORIES
            $val_categorie = null;
            $cellCategorieStart = null;
            $cellCategorieEnd = null;
            if (!empty($columnCATEGORIES) &&
                !empty($rowCATEGORIES) &&
                !empty($columnDESIGNATION) &&
                !empty($rowDESIGNATION) &&
                $columnCATEGORIES === $columnDESIGNATION
            ) {
                $rowCategorieStart = $rowCATEGORIES + 1;
                $rowCategorieEnd = $rowDESIGNATION - 1;

                $cellCategorieStart = $columnCATEGORIES . $rowLeiciteStart;
                $cellCategorieEnd = $columnCATEGORIES . $rowLeiciteEnd;

                if ($cellCategorieStart == $cellCategorieEnd) {
                    // Une seule ligne de Leicite
                    $val_categorie = $worksheet->getCell($cellCategorieStart)->getValue();
                } else {
                    $tmpCategorie = null;
                    for ($i = $rowCategorieStart; $i <= $rowCategorieEnd; $i++) {
                        $tmpCategorie = $worksheet->getCell($columnCATEGORIES . $i)->getValue();

                        if (!empty($tmpCategorie)) {
                            $val_categorie .= $tmpCategorie;
                        }
                    }
                }
            } elseif (!empty($columnCATEGORIES) &&
                !empty($rowCATEGORIES) &&
                !empty($lastRow)
            ) {
                $rowCategorieStart = $rowCATEGORIES + 1;
                $rowCategorieEnd = $lastRow;

                $tmpCategorie = null;
                for ($i = $rowCategorieStart; $i <= $rowCategorieEnd; $i++) {
                    $tmpCategorie = $worksheet->getCell($columnCATEGORIES . $i)->getValue();

                    if (!empty($tmpCategorie)) {
                        $val_categorie .= $tmpCategorie;
                    }
                }
            } else {
                $filesErrorNoCategorie[] = [
                    'file' => $file,
                    'error' => 'ERROR CATEGORIE - NO FILE IN INSERT',
                ];
                $success = false;
            }

            if (empty($val_categorie) && $success === true) {
                $val_categorie = 'A COMPLETER - CATEGORIE VIDE';

                $filesErrorNoCategorie[] = [
                    'file' => $file,
                    'warning' => 'NO CATEGORIE',
                    'insertValue' => 'A COMPLETER - CATEGORIE VIDE'
                ];
            }

            if ($spreadsheet->sheetNameExists('DATA')) {
                $worksheetDATA = $spreadsheet->setActiveSheetIndexByName('DATA');
            } else {
                $filesErrorNoSheetExists[] = [
                    'file' => $file,
                    'feuille' => 'DATA no existe'
                ];
            }

            $fieldsToCollected_DonneesCourantes = [
                "Etat civil",
                "Vie personnelle",
                "Vie professionnelle",
                "D'ordre économique 
et financier",
                "Données de connexion",
                "Données de localisation",
                "Internet",
                "Autres catégories de données",
            ];

            $fieldsToCollected_DonneesSensibles = [
                "Origine raciale  ou ethnique",
                "Opinions politiques",
                "Convictions religieuses ou philo",
                "Appartenance syndicale",
                "Données génétiques",
                "Données biométriques",
                "Données concernant la santé",
                "Vie ou orientation sexuelle",
                "Condamnations pénale ou infractions",
                "Numéro d'identification national unique (NIR)",
            ];

            $donneesXLSX = [];

            $donneesCourantes = '';
            $donneesSensibles = [];
            foreach ($worksheetDATA->getRowIterator() as $rowDATA) {
                $cellIteratorDATA = $rowDATA->getCellIterator();

                foreach ($cellIteratorDATA as $cellDATA) {
                    $getValueFile = trim($cellDATA->getValue());

                    if (in_array($getValueFile, $fieldsToCollected_DonneesCourantes)) {
                        $data = $this->getValueDATA(
                            $getValueFile,
                            $cellDATA,
                            $worksheetDATA
                        );

                        if ($data != null) {
                            $donneesCourantes .= $data;
                        }
                    }

                    if (in_array($getValueFile, $fieldsToCollected_DonneesSensibles)) {
                        $arrayDonneesSensibles = $this->getValueDATA_donneesSensibles(
                            $getValueFile,
                            $cellDATA,
                            $worksheetDATA
                        );

                        if ($getValueFile != "Numéro d'identification national unique (NIR)") {
                            if (!empty($arrayDonneesSensibles)) {
                                $keyDonneesSensibles = count($donneesSensibles)+1;

                                $donneesSensibles[$keyDonneesSensibles] = $arrayDonneesSensibles;
                            }
                        } else {
                            $donneesXLSX = array_merge(
                                $donneesXLSX,
                                $arrayDonneesSensibles
                            );
                        }
                    }
                }
            }

            if ($spreadsheet->sheetNameExists('SECURITE')) {
                $worksheetSECURITE = $spreadsheet->setActiveSheetIndexByName('SECURITE');
            } else {
                $filesErrorNoSheetExists[] = [
                    'file' => $file,
                    'feuille' => 'SECURITE no existe'
                ];
            }

            $fieldsToCollected_CategorieDestinataires = [
                "Destinataires internes",
                "Organismes externes",
                "sous-traitants",
            ];

            $fieldsToCollected_MesuresTechnique = [
                "Contrôle d'accès des utilisateurs",
                "Mesures de traçabilité",
                "Mesures de Protection des logiciels",
                "Sauvegarde des données",
                "Chiffrement des données",
                "Contrôle des sous-traitants",
                "Autes mesures",
            ];

//Contrôle d'accès des utilisateurs = Contrôle des accès logiques
//
//Mesures de traçabilité =  Traçabilité (journalisation)
//
//Mesures de Protection des logiciels = Lutte contre les logiciels malveillants
//
//Sauvegarde des données = Sauvegardes
//
//Chiffrement des données = Chiffrement
//
//Contrôle des sous-traitants = Relations avec les tiers ( AJOUTER ONGLET)
//
//Autes mesures = Autes mesures technique

            $donneesTransfertDonnees = [];
            foreach ($worksheetSECURITE->getRowIterator() as $rowSECURITE) {
                $cellIteratorSECURITE = $rowSECURITE->getCellIterator();

                foreach ($cellIteratorSECURITE as $cellSECURITE) {
                    $getValueFile = trim($cellSECURITE->getValue());

                    if (in_array($getValueFile, $fieldsToCollected_CategorieDestinataires)) {
                        $donneesXLSX = array_merge(
                            $donneesXLSX,
                            $this->getValueSECURITE_categorieDestinataire(
                                $getValueFile,
                                $cellSECURITE,
                                $worksheetSECURITE
                            )
                        );
                    }

                    if (in_array($getValueFile, $fieldsToCollected_MesuresTechnique)) {
                        $donneesXLSX = array_merge(
                            $donneesXLSX,
                            $this->getValueSECURITE_mesuresTechnique(
                                $getValueFile,
                                $cellSECURITE,
                                $worksheetSECURITE
                            )
                        );
                    }

//                    if ($getValueFile == 'Si oui vers quel pays et
//Préciser les types de garanties') {
//                        $donneesTransfertDonnees[] = $this->getValueSECURITE_transfertDonnees(
//                            $getValueFile,
//                            $cellSECURITE,
//                            $worksheetSECURITE
//                        );
//                    }
                }
            }

            if ($success === true) {
                $ficheDonnees_sensibles = '0';
                if (!empty($donneesSensibles)) {
                    $donneesXLSX = array_merge($donneesXLSX, [
                        'donneessensibles' => $donneesSensibles,
                    ]);

                    $ficheDonnees_sensibles = '1';
                }

                $infoFiles[$key] = [
                    'file' => $file,
                    'submit' => 'Save',
                    'WebdpoFiche' => array_merge(
                        [
                            'declarantpersonnenom' => 'Théo GUILLON',
                            'declarantservice' => 'Aucun service',
                            'declarantpersonneemail' => 'theo.guillon@libriciel.coop',
                            'outilnom' => $titleFiche,
                            'finaliteprincipale' => $val_finalite,
                            'det_liceite' => $val_leicite,
                            'autre_donnes_courantes' => $donneesCourantes,
                        ],
                        $donneesXLSX
                    ),
                    'Fiche' => [
                        'transfert_hors_ue' => '0',
                        'donnees_sensibles' => $ficheDonnees_sensibles,
                        'realisation_pia' => '0',
                        'coresponsable' => '0',
                        'soustraitance' => '0',
                    ],
                    'Coresponsable' => [
                        'coresponsables' => ''
                    ],
                    'Soustraitance' => [
                        'soustraitances' => ''
                    ],
//                    'info_traitement' => [
//                        'created' => $createdFiche,
//                        'modified' => $modifiedFiche,
//                    ],
//                    'valeur_traitement' => [
//                        'val_categorie' => $val_categorie,
//                    ],
                ];

//                $result = $this->requestAction([
//                    'plugin' => null,
//                    'controller' => 'fiches',
//                    'action' => 'initialisation',
//                    1
//                ], [
//                    'data' => $infoFiles[$key]
//                ]);
//
//                if ($result === false) {
//                    debug("ERROR");
//                    die;
//                }
            }
        }

        debug($infoFiles);

        debug($filesErrorNoSheetExists);
        debug($filesErrorTitle);
        debug($filesErrorNoFinalite);
        debug($filesErrorNoLeicite);
        debug($filesErrorNoCategorie);

        die;
    }

    protected function getValueDATA($getValueFile, $cellDATA, $worksheetDATA)
    {
        $val = null;
        $valueField = false;
        $description = '';
        $exportToFileDescription = [];

        $rowDATA = $cellDATA->getRow();
        foreach (range('B','D') as $columnDATA) {
            $tmpDescription = trim($worksheetDATA->getCell($columnDATA.$rowDATA)->getValue());

            $exportToFileDescription[$columnDATA.$rowDATA] = $tmpDescription;

            if (!empty($tmpDescription)) {
                if ($tmpDescription == 'oui' ||
                    $tmpDescription == 'OUI' ||
                    $tmpDescription == 'Oui'
                ) {
                    $valueField = true;
                    $description .= '';
                } elseif ($tmpDescription == 'non' ||
                    $tmpDescription == 'NON' ||
                    $tmpDescription == 'Non' ||
                    $tmpDescription == 'Néant' ||
                    $tmpDescription == 'néant'
                ) {
                    $valueField = false;
                    $description = '';
                } else {
                    $valueField = true;
                    $description .= $tmpDescription;
                }
            }
        }

        if ($valueField === true) {
            $val = $getValueFile . ' = ' . $description . ' | ';
        }

        return $val;
    }

    protected function getValueDATA_donneesSensibles($getValueFile, $cellDATA, $worksheetDATA)
    {
        $val = [];
        $valueField = false;
        $description = '';
        $effacement = '';

        $rowDATA = $cellDATA->getRow();
        foreach (range('B','D') as $columnDATA) {
            $tmpDescription = trim($worksheetDATA->getCell($columnDATA.$rowDATA)->getValue());

            if (!empty($tmpDescription)) {
                if ($tmpDescription == 'oui' ||
                    $tmpDescription == 'OUI' ||
                    $tmpDescription == 'Oui'
                ) {
                    $valueField = true;
                    $description .= '';
                } elseif ($tmpDescription == 'non' ||
                    $tmpDescription == 'NON' ||
                    $tmpDescription == 'Non' ||
                    $tmpDescription == 'Néant' ||
                    $tmpDescription == 'néant' ||
                    $tmpDescription == 'nénat' ||
                    $tmpDescription == 'X' ||
                    $tmpDescription == 'n/a' ||
                    $tmpDescription == 'RAS' ||
                    $tmpDescription == 'Sans Objet' ||
                    $tmpDescription == 'Sans objet' ||
                    $tmpDescription == '/'
                ) {
                    $valueField = false;
                    $description = '';
                } else {
                    $valueField = true;
                    $description .= $tmpDescription;
                }
            }
        }

        foreach (range('E','F') as $columnEffacement) {
            $tmpEffacement = trim($worksheetDATA->getCell($columnEffacement.$rowDATA)->getValue());

            if (!empty($tmpEffacement)) {
                $effacement .= $tmpEffacement;
            }
        }

        $newNameFieldForm = [
            'origine_raciale_ou_ethnique' => 'Données révélant l\'origine raciale ou ethnique',
            'opinions_politiques' => 'Données révélant les opinions politiques',
            'convictions_religieuses_ou_philo' => 'Données révélant les convictions religieuses ou philosophiques ',
            'appartenance_syndicale' => 'Données révélant l\'appartenance syndicale',
            'donnees_genetiques' => 'Données génétiques',
            'donnees_biometriques' => 'Données biométriques aux fins d\'identifier une personne physique de manière unique',
            'donnees_concernant_la_sante' => 'Données concernant la santé',
            'vie_ou_orientation_sexuelle' => 'Données concernant la vie sexuelle ou l\'orientation sexuelle',
            'condamnations_penale_ou_infractions' => 'Données relatives à des condamnations pénales ou infractions',
//            'numero_didentification_national_unique_nir' => 'Le numéro de sécurité sociale (NIR)',
        ];
        $formatageVariableXLSX = $this->formatageVariable($getValueFile);

        if ($formatageVariableXLSX != 'numero_didentification_national_unique_nir') {
            if ($valueField === false) {
                return $val;
            }

            $field_id = $newNameFieldForm[$formatageVariableXLSX];

            $val = [
                'typeDonneeSensible' => $field_id,
                'descriptionDonneeSensible' => $description,
                'dureeConservationDonneeSensible' => $effacement
            ];
        } else {
            if ($valueField === true) {
                $use_nir = 'Oui';
            } else {
                $use_nir = 'Non';
            }

            $val = [
                'nir_collecte' => $use_nir,
                'nir_description' => $description,
                'nir_effacement' => $effacement
            ];
        }

        return $val;
    }

    protected function getValueSECURITE_categorieDestinataire($getValueFile, $cellSECURITE, $worksheetSECURITE)
    {
        $useField = null;

        $rowSECURITE = $cellSECURITE->getRow();

        $description = trim($worksheetSECURITE->getCell('B'.$rowSECURITE)->getValue());

        if (!empty($description) &&
            $description != '?'
        ) {
            $useField = true;
        } else {
            $useField = false;
            $description = '';
        }

        $vecteurCommunication = trim($worksheetSECURITE->getCell('C'.$rowSECURITE)->getValue());
        if ($vecteurCommunication == '?'
        ) {
            $vecteurCommunication = '';
        }

        $materielConsultation = trim($worksheetSECURITE->getCell('D'.$rowSECURITE)->getValue());
        if ($materielConsultation == '?'
        ) {
            $materielConsultation = '';
        }

        $newNameFieldForm = [
            'destinataires_internes' => 'destinataires_internes',
            'organismes_externes' => 'destinataires_externes',
            'sous-traitants' => 'sous_traitants',
        ];

        $formatageVariableXLSX = $this->formatageVariable($getValueFile);
        $field_id = $newNameFieldForm[$formatageVariableXLSX];

        if ($field_id !== 'sous_traitants') {
            if ($useField === true) {
                $valUseField = 'Oui';
            } else {
                $valUseField = 'Non';
            }

            $val = [
//                'field' => $getValueFile,
                $field_id => $valUseField,
                $field_id . '_description' => $description,
                $field_id . '_vecteur_communication' => $vecteurCommunication,
                $field_id . '_materiel_consultation' => $materielConsultation,
            ];

        } else {
            $val = [
//                'field' => $getValueFile,
                $field_id => $description,
            ];
        }

        return $val;
    }

    protected function getValueSECURITE_mesuresTechnique($getValueFile, $cellSECURITE, $worksheetSECURITE)
    {
        $useField = null;

        $rowSECURITE = $cellSECURITE->getRow();

        $description = trim($worksheetSECURITE->getCell('B'.$rowSECURITE)->getValue());

        if (!empty($description)) {
            if ($description == 'oui' ||
                $description == 'OUI' ||
                $description == 'Oui'
            ) {
                $useField = true;
                $description = '';
            } elseif ($description == 'non' ||
                $description == 'NON' ||
                $description == 'Non' ||
                $description == 'Néant' ||
                $description == 'néant'
            ) {
                $useField = false;
                $description = '';
            } else {
                $useField = true;
            }
        } else {
            $useField = false;
            $description = '';
        }

        $newNameFieldForm = [
            'controle_dacces_des_utilisateurs' => 'mesure_technique_controle_acces_logique',
            'mesures_de_tracabilite' => 'mesure_technique_tracabilite',
            'mesures_de_protection_des_logiciels' => 'mesure_technique_logiciels_malveillants',
            'sauvegarde_des_donnees' => 'mesure_technique_sauvegardes',
            'chiffrement_des_donnees' => 'mesure_technique_chiffrement',
            'controle_des_sous-traitants' => 'mesure_organisationnelle_relations_tiers',
            'autes_mesures' => 'mesure_technique_autres_mesures',
        ];

        $formatageVariableXLSX = $this->formatageVariable($getValueFile);
        $field_id = $newNameFieldForm[$formatageVariableXLSX];

        if ($useField === true) {
            $valUseField = 'Oui';
        } else {
            $valUseField = 'Non';
        }

        $val = [
//            'field' => $getValueFile,
            $field_id => $valUseField,
            $field_id.'_description' => $description,
        ];

        if ($field_id === 'mesure_technique_tracabilite') {
            $effacement = trim($worksheetSECURITE->getCell('C'.$rowSECURITE)->getValue());

            $val += [
                $field_id.'_effacement' => $effacement,
            ];
        }

        return $val;
    }

    protected function getValueSECURITE_transfertDonnees($getValueFile, $cellSECURITE, $worksheetSECURITE)
    {
        $val = null;
        $useField = null;

        $rowSECURITE = $cellSECURITE->getRow();

        $description = trim($worksheetSECURITE->getCell('B'.$rowSECURITE)->getValue());

        if (!empty($description)) {
            if ($description == 'oui' ||
                $description == 'OUI' ||
                $description == 'Oui'
            ) {
                $useField = true;
                $description = '';
            } elseif ($description == 'non' ||
                $description == 'NON' ||
                $description == 'Non' ||
                $description == 'Néant' ||
                $description == 'néant' ||
                $description == 'NC' ||
                $description == 'Aucun'
            ) {
                $useField = false;
                $description = '';
            } else {
                $useField = true;
            }
        } else {
            $useField = false;
            $description = '';
        }

        if ($useField === true) {
            $val = [
                'field' => $getValueFile,
                [
                    'champ_name' => $this->formatageVariable($getValueFile),
                    'valeur' => $useField,
                ],
                [
                    'champ_name' => $this->formatageVariable($getValueFile) . '_description',
                    'valeur' => $description,
                ],
            ];
        }

        return $val;
    }

    public function formatageVariable($valeur)
    {
        // valeur sans espace en trop dans la chaine
        $valeur = preg_replace('/\s\s+/', ' ', $valeur);

        // valeur sans accents
        $valeur = $this->_noAccents($valeur);

        // valeur en minuscule
        $valeur = mb_convert_case($valeur, MB_CASE_LOWER);

        // supprime espace en début et fin de la chaine
        $valeur = trim($valeur);

        // remplace les espaces par _
        $valeur = str_replace(' ', '_', $valeur);

        // remplace les caractères spéciaux
        $valeur = preg_replace('#[^a-zA-Z 0-9_-]#', '', $valeur);

        return ($valeur);
    }

    protected function _noAccents($str)
    {
        $str = htmlentities($str, ENT_COMPAT, 'UTF-8');
        $ligs = array(
            'from' => array('&aelig;', '&AElig;', '&oelig;', '&OElig;', '&szlig;'),
            'to' => array('ae', 'AE', 'oe', 'OE', 's')
        );
        $str = str_replace($ligs['from'], $ligs['to'], $str);
        $symbols = 'uml|acute|grave|circ|tilde|cedil|ring|th|slash|horn';
        $str = mb_ereg_replace('&([a-zA-Z])('.$symbols.');', '\1', $str);
        return html_entity_decode($str, ENT_COMPAT, 'UTF-8');
    }

    function startsWith( $haystack, $needle ) {
        $length = strlen( $needle );
        return substr( $haystack, 0, $length ) === $needle;
    }
}
